package com.galaxis.manatee.controller;

import com.galaxis.manatee.dao.ChuanyunUserCompanyDao;
import com.galaxis.manatee.entity.chuanyun.data.object.ChuanyunUserCompanyDO;
import com.galaxis.manatee.service.LogCheckService;
import com.galaxis.manatee.service.LogService;
import com.galaxis.manatee.service.LogStandardService;
import com.galaxis.manatee.service.LogUpdateService;
import com.galaxis.manatee.task.WorkHourStatistics;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.GetMapping;
import org.springframework.web.bind.annotation.RequestParam;
import org.springframework.web.bind.annotation.RestController;

import java.util.List;
import java.util.stream.Collectors;

/**
 * 日志工时controller
 *
 * @author zcj
 * @version 0.1
 * @date 2021/3/4 3:56 上午
 */
@Slf4j
@RestController
public class WorkHourController {
    @Autowired
    private ChuanyunUserCompanyDao userCompanyDao;
    private final LogCheckService logCheckService;
    private final LogStandardService logStandardService;
    private final LogUpdateService logUpdateService;
    private final WorkHourStatistics workHourStatistics;
    private final LogService logService;

    public WorkHourController(LogCheckService logCheckService, WorkHourStatistics workHourStatistics, LogStandardService logStandardService, LogUpdateService logUpdateService, LogService logService) {
        this.logCheckService = logCheckService;
        this.workHourStatistics = workHourStatistics;
        this.logStandardService = logStandardService;
        this.logUpdateService = logUpdateService;
        this.logService = logService;
    }

    /**
     * 检查工时不一致数据
     *
     * @param userId 用户Id
     */
    @GetMapping("/test/workHour/checkSelfWorkHour")
    public void checkSelfWorkHour(@RequestParam("userId") String userId) {
        try {
            logCheckService.checkDayWorkHour(userId);
        } catch (Exception e) {
            e.printStackTrace();
        }
    }

    /**
     * 标准化全部工时信息
     */
    @GetMapping("/test/workHour/standardDayWorkHourApi")
    public void standardDayWorkHourApi() {
        workHourStatistics.standardDayWorkHour();
    }

    /**
     * 上传日工时至氚云数据库
     */
    @GetMapping("/test/workHour/pullDayWorkHourApi")
    public void pullDayWorkHourApi() {
        workHourStatistics.pushDayWorkHour();
    }

    /**
     * 标准化半年内工时信息
     */
    @GetMapping("/test/workHour/standardHalfYearDayWorkHourApi")
    public void standardHalfYearDayWorkHourApi() {
        log.info("手动标准化每日工时信息");
        logStandardService.standardHalfYearWorkHour();
    }

    /**
     * 标准化半年内工时信息
     */
    @GetMapping("/test/workHour/standardHalfYearDayWorkHourApiByUserId")
    public void standardHalfYearDayWorkHourApiByUserId(@RequestParam("userId") String userId) {
        logStandardService.standardHalfYearWorkHourByUserId(userId);
    }

    /**
     * 上传半年内日工时至氚云数据库
     */
    @GetMapping("/test/workHour/pullHalfYearDayWorkHourApi")
    public void pullHalfYearDayWorkHourApi() {
        logService.updateHalfYearChuanyunSelfWorkHour();
    }

    /**
     * 标准化一年内周工时和月工时
     */
    @GetMapping("/test/workHour/standardWeekAndMonth")
    public void standardWeekAndMonth() {
        workHourStatistics.standardWeekAndMonthWorkHour();
    }

    /**
     * 上传一年内周工时和月工时工时至氚云数据库
     */
    @GetMapping("/test/workHour/pushWeekAndMonth")
    public void pushWeekAndMonth() {
        workHourStatistics.pushWorkHour();
    }

    /**
     * 检验一个季度的工时日志
     */
    @GetMapping("/test/workHour/checkQuarterDayWorkHour")
    public void checkQuarterDayWorkHour() {
        List<String> userIdList = userCompanyDao.findAll().stream().map(ChuanyunUserCompanyDO::getUserId).collect(Collectors.toList());
        List<String> disUserIdList = userIdList.stream().distinct().collect(Collectors.toList());
        logCheckService.checkDayWorkHourTimeAfter(disUserIdList, 90);
    }

    /**
     * 校验指定日期氚云数据库的工时记录是否存在+8时区的数据
     *
     * @param year   年
     * @param month  月
     * @param maxDay 当月最后一天
     */
    @GetMapping("/test/api/checkAllMemberHour")
    public void checkMemberHour(@RequestParam("year") Integer year, @RequestParam("month") Integer month, @RequestParam("maxDay") Integer maxDay) {
        logCheckService.checkChuanYunAll(year, month, maxDay);
    }

    /**
     * 标准化全部工时信息
     */
    @GetMapping("/test/workHour/standardAllDayWorkHourApi")
    public void standardAllDayWorkHourApi() {
        workHourStatistics.standardAllWorkHour();
    }

    /**
     * 标准化全部工时信息
     */
    @GetMapping("/test/workHour/pushAllWorkHourApi")
    public void pushAllWorkHourApi() {
        workHourStatistics.pushAllWorkHour();
    }
}
