package com.galaxis.manatee.service;

import com.fasterxml.jackson.core.type.TypeReference;
import com.fasterxml.jackson.databind.ObjectMapper;
import com.galaxis.manatee.constant.ChuanYunConstant;
import com.galaxis.manatee.dao.*;
import com.galaxis.manatee.entity.chuanyun.data.object.*;
import com.galaxis.manatee.entity.chuanyun.dto.Filter;
import com.galaxis.manatee.manager.ChuanYunManager;
import com.galaxis.manatee.util.TimeUtil;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.util.StringUtils;

import java.time.Instant;
import java.time.LocalDateTime;
import java.time.format.DateTimeFormatter;
import java.time.temporal.ChronoUnit;
import java.util.ArrayList;
import java.util.List;
import java.util.Objects;
import java.util.Optional;

/**
 * 下载日志原始数据
 *
 * @author zcj
 * @version 0.1
 * @date 2021/3/13 2:14 上午
 */
@Slf4j
@Service
public class LogDownloadService {

    private WorkGroupService workGroupService;

    private ChuanyunProjectDao chuanyunProjectDao;
    private ChuanyunFactoryLocationDao factoryLocationDao;
    private ChuanyunEquipmentProcessDao equipmentProcessDao;

    private final ChuanYunManager chuanYunManager;
    private final ChuanyunLogDao chuanyunLogDao;
    private final ChuanyunUserCompanyDao userCompanyDao;
    private final ChuanyunGroupProjectDao chuanyunGroupProjectDao;
    private final ChuanyunUserDepartmentNameDao chuanyunUserDepartmentNameDao;

    public LogDownloadService(ChuanYunManager chuanYunManager, ChuanyunLogDao chuanyunLogDao, ChuanyunUserCompanyDao chuanyunUserCompanyDao, ChuanyunGroupProjectDao chuanyunGroupProjectDao, ChuanyunUserDepartmentNameDao chuanyunUserDepartmentNameDao) {
        this.chuanYunManager = chuanYunManager;
        this.chuanyunLogDao = chuanyunLogDao;
        this.userCompanyDao = chuanyunUserCompanyDao;
        this.chuanyunGroupProjectDao = chuanyunGroupProjectDao;
        this.chuanyunUserDepartmentNameDao = chuanyunUserDepartmentNameDao;
    }

    @Autowired
    public void setWorkGroupService(WorkGroupService workGroupService) {
        this.workGroupService = workGroupService;
    }

    @Autowired
    public void setChuanyunProjectDao(ChuanyunProjectDao chuanyunProjectDao) {
        this.chuanyunProjectDao = chuanyunProjectDao;
    }

    @Autowired
    public void setFactoryLocationDao(ChuanyunFactoryLocationDao factoryLocationDao) {
        this.factoryLocationDao = factoryLocationDao;
    }

    @Autowired
    public void setEquipmentProcessDao(ChuanyunEquipmentProcessDao equipmentProcessDao) {
        this.equipmentProcessDao = equipmentProcessDao;
    }

    /**
     * 更新日志数据
     *
     * @param isRecent 是否只更新最近30天
     */
    public void saveChuanyunLogDO(boolean isRecent) {
        var objectMapper = new ObjectMapper();
        var start = 0;
        var totalCount = 0L;
        var flag = true;
        var pageSize = 100;
        List<String> matchersString = new ArrayList<>();
        //由于需要统计流程中的日志，在日志数据同步时将状态为2的数据一并统计
        matchersString.add("Status_6,[1;2;3]");
        // 默认获取半年的日志
        DateTimeFormatter df = DateTimeFormatter.ofPattern("yyyy-MM-dd HH:mm:ss");
        // 按要求同步两个月的记录
        LocalDateTime localDateTime = LocalDateTime.now().minusDays(90);
        matchersString.add("logDate_1," + df.format(localDateTime));
        if (isRecent) {
            //获取当前时间30天前日期
            LocalDateTime now = LocalDateTime.now();
            now = now.minus(30, ChronoUnit.DAYS);
            matchersString.add("logDate_1," + df.format(now));
        }
        while (flag) {
            long startTime = Instant.now().getEpochSecond();
            try {
                //从氚云查询数据
                var filter = Filter.instance(start, start + pageSize, true, Filter.AND, matchersString);
                var chuanyunFindAllResponse = chuanYunManager.findAll(ChuanyunLogDO.SCHEMA_CODE, filter);
                if (chuanyunFindAllResponse.getReturnData() == null) {
                    flag = false;
                    continue;
                }
                if (chuanyunFindAllResponse.getReturnData().getTotalCount() >= start + pageSize) {
                    start += pageSize;
                } else {
                    flag = false;
                }
                //转化为POJO
                List<ChuanyunLogDO> result = objectMapper.convertValue(chuanyunFindAllResponse.getReturnData().getBizObjectArray(), new TypeReference<>() {
                });
                //处理日志数据
                result.forEach(this::updateChuanyunLogData);
                //获取项目总数
                totalCount = chuanyunFindAllResponse.getReturnData().getTotalCount();
                //保存
                chuanyunLogDao.saveAllAndFlush(result);
            } catch (Exception e) {
                log.error(e.getMessage());
                e.printStackTrace();
            }
            log.info("日志100条保存时间" + (Instant.now().getEpochSecond() - startTime) + "秒");
        }
        log.info("最近一月历史日志信息保存" + totalCount);
    }

    /**
     * 更新氚云日志数据
     *
     * @param chuanyunLogDO 待处理数据
     */
    public void updateChuanyunLogData(ChuanyunLogDO chuanyunLogDO) {
        //处理项目userId
        if (chuanyunLogDO.getUserObject() != null) {
            chuanyunLogDO.setUserId(chuanyunLogDO.getUserObject().getObjectId());
        }
        //日志人员所属部门名称
        if (!StringUtils.isEmpty(chuanyunLogDO.getDepartment())) {
            chuanyunLogDO.setDepartmentName(chuanyunLogDO.getDepartment());
        } else {
            chuanyunLogDao.findById(chuanyunLogDO.getObjectId()).ifPresent(oldLogDO -> {
                chuanyunLogDO.setDepartmentName(oldLogDO.getDepartmentName());
            });
        }

        // 补全部门信息
        if (Objects.isNull(chuanyunLogDO.getDepartmentName()) || chuanyunLogDO.getDepartmentName().isBlank()) {
            // 任没有部门，查询该用户所属公司获取部门
            Optional<ChuanyunUserCompanyDO> first = userCompanyDao.findAllByUserIdOrderByModifiedTimeDesc(chuanyunLogDO.getUserId()).stream().findFirst();
            first.ifPresent(chuanyunUserCompanyDO -> {
                String[] split = chuanyunUserCompanyDO.getDepartment().split("/");
                int length = split.length;
                chuanyunLogDO.setDepartmentName(split[length - 1]);
            });
        }
        //处理数据
        if (chuanyunLogDO.getDayLogList() == null || chuanyunLogDO.getDayLogList().size() == 0) {
            chuanyunLogDO.setDayLogList(new ArrayList<>());
            log.warn(chuanyunLogDO + "");
            log.warn("异常数据");
        } else {
            //如果有重复就删除
            chuanyunLogDao.findById(chuanyunLogDO.getObjectId()).ifPresent(chuanyunLogDao::delete);
            //根据projectId获取code,如果是生产项目，则不用处理
            //针对前期测试数据，可能出现日志为空的情况，跳过不处理
            chuanyunLogDO.getDayLogList().forEach(chuanyunDayLog -> {
                // 补全工作组
                if (ChuanYunConstant.NEED_WORK_GROUP_PROJECT_TYPE.contains(chuanyunDayLog.getProjectType())) {
                    if (ChuanYunConstant.LOG_WEEK_TYPE.contains(chuanyunLogDO.getDayOrWeek())) {
                        List<ChuanyunWeekLogDO> weekLogList = chuanyunLogDO.getWeekLogList();
                        for (ChuanyunWeekLogDO chuanyunWeekLogDO : weekLogList) {
                            if (Objects.equals(chuanyunDayLog.getProjectId(), chuanyunWeekLogDO.getProjectId())) {
                                String workGroupId = chuanyunWeekLogDO.getWorkGroupId();
                                if (StringUtils.hasLength(workGroupId)) {
                                    chuanyunDayLog.setWorkGroupId(workGroupId);
                                    ChuanyunWorkGroupDO workGroupDO = workGroupService.findByObjectId(workGroupId);
                                    if (!Objects.isNull(workGroupDO)) {
                                        chuanyunDayLog.setWorkGroupName(workGroupDO.getWorkGroupName());
                                    } else {
                                        ChuanyunHistoryWorkGroupDO historyWorkGroup = workGroupService.findInHistoryByObjectId(workGroupId);
                                        if (!Objects.isNull(historyWorkGroup)) {
                                            chuanyunDayLog.setWorkGroupName(historyWorkGroup.getWorkGroupName());
                                        }
                                    }
                                    break;
                                }
                            }
                        }
                    } else if (ChuanYunConstant.LOG_FILL_IN_DAY.contains(chuanyunLogDO.getDayOrWeek()) && StringUtils.hasLength(chuanyunDayLog.getWorkGroupId())) {
                        ChuanyunWorkGroupDO workGroupDO = workGroupService.findByObjectId(chuanyunDayLog.getWorkGroupId());
                        if (!Objects.isNull(workGroupDO)) {
                            chuanyunDayLog.setWorkGroupName(workGroupDO.getWorkGroupName());
                        } else {
                            ChuanyunHistoryWorkGroupDO historyWorkGroup = workGroupService.findInHistoryByObjectId(chuanyunDayLog.getWorkGroupId());
                            if (!Objects.isNull(historyWorkGroup)) {
                                chuanyunDayLog.setWorkGroupName(historyWorkGroup.getWorkGroupName());
                            }
                        }
                    }

                    if (!StringUtils.hasLength(chuanyunDayLog.getWorkGroupId())) {
                        // 赋予默认工作组
                        chuanyunDayLog.setWorkGroupId(ChuanYunConstant.DEFAULT_WORK_GROUP);
                        chuanyunDayLog.setWorkGroupName(ChuanYunConstant.DEFAULT_WORK_GROUP_NAME);
                    }
                }


                //离职人员补全用户Id及部门ID
                if (StringUtils.isEmpty(chuanyunLogDO.getUserId())) {
                    var name = "";
                    if (StringUtils.isEmpty(chuanyunDayLog.getContent())) {
                        name = chuanyunLogDO.getName();
                    } else {
                        name = chuanyunDayLog.getContent();
                    }
                    chuanyunLogDO.setUserId(getCharacters(name));
                    ChuanyunUserDepartmentNameDO chuanyunUserDepartmentNameDO = chuanyunUserDepartmentNameDao.findByUserName(chuanyunLogDO.getUserId());
                    if (chuanyunUserDepartmentNameDO != null) {
                        chuanyunLogDO.setDepartmentName(chuanyunUserDepartmentNameDO.getDepartmentName());
                    }
                }
                try {
                    if (!ChuanYunConstant.PRODUCTION_PROJECT.equals(chuanyunDayLog.getProjectType()) && StringUtils.isEmpty(chuanyunDayLog.getProjectId())) {
                        log.info(chuanyunLogDO + "");
                    } else if (!ChuanYunConstant.PRODUCTION_PROJECT.equals(chuanyunDayLog.getProjectType())) {
                        Optional<ChuanyunGroupProjectDO> gpOptional = chuanyunGroupProjectDao.findById(chuanyunDayLog.getProjectId());
                        if (gpOptional.isPresent()) {
                            chuanyunDayLog.setProjectCode(gpOptional.get().getProjectCode());
                        } else {
                            // 查询总况表
                            Optional<ChuanyunProjectDO> projectDO = chuanyunProjectDao.findById(chuanyunDayLog.getProjectId());
                            projectDO.ifPresent(chuanyunProjectDO -> chuanyunDayLog.setProjectCode(chuanyunProjectDO.getProjectCode()));
                        }
                    }
                    if (Objects.equals(ChuanYunConstant.PRODUCTION_PROJECT, chuanyunDayLog.getProjectType())) {
                        String factoryLocationId = chuanyunDayLog.getFactoryLocationId();
                        Optional<ChuanyunFactoryLocation> factoryLocationOptional = factoryLocationDao.findById(factoryLocationId);
                        factoryLocationOptional.ifPresent(chuanyunFactoryLocation -> chuanyunDayLog.setFactoryLocationName(chuanyunFactoryLocation.getFactoryName()));
                        String jobContentId = chuanyunDayLog.getJobContentId();
                        Optional<ChuanyunEquipmentProcess> equipmentProcessOptional = equipmentProcessDao.findById(jobContentId);
                        equipmentProcessOptional.ifPresent(chuanyunEquipmentProcess -> chuanyunDayLog.setJobContent(chuanyunEquipmentProcess.getProcedureInfo()));
                    }

                } catch (Exception e) {
                    e.printStackTrace();
                    log.warn(chuanyunDayLog + "");
                }
            });
        }

    }

    private String getCharacters(String str) {
        String reg = "[^\u4e00-\u9fa5]";
        str = str.replaceAll(reg, " ");
        return str.trim();
    }

    private String getDepartmentName(String str) {
        if (str.contains("/")) {
            String[] department = str.split("/");
            return department[department.length - 1];
        } else {
            return str;
        }
    }


    /**
     * 更新日志数据
     */
    public void saveDayLog() {
        var objectMapper = new ObjectMapper();
        var start = 0;
        var totalCount = 0L;
        var flag = true;
        var pageSize = 100;
        List<String> matchersString = new ArrayList<>();
        //由于需要统计流程中的日志，在日志数据同步时将状态为2的数据一并统计
        matchersString.add("Status_6,[1;2;3]");
        LocalDateTime localDateTimeStart = LocalDateTime.of(2022, 8, 1, 0, 0, 0);
        LocalDateTime localDateTimeEnd = LocalDateTime.of(2022, 8, 30, 23, 59, 59);
        matchersString.add("logDate_1," + TimeUtil.formatDateTime(localDateTimeStart));
        matchersString.add("logDate_3," + TimeUtil.formatDateTime(localDateTimeEnd));
        while (flag) {
            long startTime = Instant.now().getEpochSecond();
            try {
                //从氚云查询数据
                var filter = Filter.instance(start, start + pageSize, true, Filter.AND, matchersString);
                var chuanyunFindAllResponse = chuanYunManager.findAll(ChuanyunLogDO.SCHEMA_CODE, filter);
                if (chuanyunFindAllResponse.getReturnData() == null) {
                    flag = false;
                    continue;
                }
                if (chuanyunFindAllResponse.getReturnData().getTotalCount() >= start + pageSize) {
                    start += pageSize;
                } else {
                    flag = false;
                }
                //转化为POJO
                List<ChuanyunLogDO> result = objectMapper.convertValue(chuanyunFindAllResponse.getReturnData().getBizObjectArray(), new TypeReference<>() {
                });
                //处理日志数据
                result.forEach(this::updateChuanyunLogData);
                //获取项目总数
                totalCount = chuanyunFindAllResponse.getReturnData().getTotalCount();
                //保存
                chuanyunLogDao.saveAll(result);
            } catch (Exception e) {
                log.error(e.getMessage());
                e.printStackTrace();
            }
            log.info("日志100条保存时间" + (Instant.now().getEpochSecond() - startTime) + "秒");
        }
        log.info("最近一月历史日志信息保存" + totalCount);
    }

    public void saveAllLog() {
        var objectMapper = new ObjectMapper();
        var start = 0;
        var totalCount = 0L;
        var flag = true;
        var pageSize = 100;
        List<String> matchersString = new ArrayList<>();
        matchersString.add("Status_6,[1;2;3]");
        while (flag) {
            long startTime = Instant.now().getEpochSecond();
            try {
                //从氚云查询数据
                var filter = Filter.instance(start, start + pageSize, true, Filter.AND, matchersString);
                var chuanyunFindAllResponse = chuanYunManager.findAll(ChuanyunLogDO.SCHEMA_CODE, filter);
                if (chuanyunFindAllResponse.getReturnData() == null) {
                    flag = false;
                    continue;
                }
                if (chuanyunFindAllResponse.getReturnData().getTotalCount() >= start + pageSize) {
                    start += pageSize;
                } else {
                    flag = false;
                }
                //转化为POJO
                List<ChuanyunLogDO> result = objectMapper.convertValue(chuanyunFindAllResponse.getReturnData().getBizObjectArray(), new TypeReference<>() {
                });
                //处理日志数据
                result.forEach(this::updateChuanyunLogData);
                //获取项目总数
                totalCount = chuanyunFindAllResponse.getReturnData().getTotalCount();
                //保存
                chuanyunLogDao.saveAll(result);
            } catch (Exception e) {
                log.error(e.getMessage());
                e.printStackTrace();
            }
            log.info("日志100条保存时间" + (Instant.now().getEpochSecond() - startTime) + "秒");
        }
        log.info("最近一月历史日志信息保存" + totalCount);
    }


}
