package com.galaxis.manatee.Test;

import com.fasterxml.jackson.annotation.JsonProperty;
import com.fasterxml.jackson.core.JsonProcessingException;
import com.fasterxml.jackson.core.type.TypeReference;
import com.fasterxml.jackson.databind.ObjectMapper;
import com.galaxis.manatee.dao.ChuanyunGroupProjectDao;
import com.galaxis.manatee.entity.chuanyun.data.object.ChuanyunGroupProjectDO;
import com.galaxis.manatee.entity.chuanyun.dto.ChuanyunFindAllBizDTO;
import com.galaxis.manatee.entity.chuanyun.dto.ChuanyunSaveDTO;
import com.galaxis.manatee.entity.chuanyun.dto.Filter;
import com.galaxis.manatee.manager.ChuanYunManager;
import lombok.Data;
import lombok.extern.slf4j.Slf4j;
import org.junit.jupiter.api.Test;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.boot.test.context.SpringBootTest;

import java.util.*;
import java.util.stream.Collectors;

@Slf4j
@SpringBootTest
public class GetRepeatProject {

    @Autowired
    private ChuanyunGroupProjectDao chuanyunGroupProjectDao;
    @Autowired
    private ChuanYunManager chuanYunManager;

    @Test
    public void getData() {
        var objectMapper = new ObjectMapper();
        var start = 0;
        var totalCount = 0L;
        var flag = true;
        var pageSize = 200;

        while (flag) {

            //从氚云查询数据
            var filter = Filter.instance(start, start + pageSize, true);
            ChuanyunFindAllBizDTO<Object> chuanyunFindAllResponse = null;
            try {
                chuanyunFindAllResponse = chuanYunManager.findAll(ChuanyunGroupProjectDO.SCHEMA_CODE, filter);
            } catch (JsonProcessingException e) {
                e.printStackTrace();
            }
            if (chuanyunFindAllResponse.getReturnData() == null) {
                flag = false;
                continue;
            }
            if (chuanyunFindAllResponse.getReturnData().getTotalCount() >= start + pageSize) {
                start += pageSize;
            } else {
                flag = false;
            }


            //转化为POJO
            List<ChuanyunGroupProjectDO> result = objectMapper.convertValue(chuanyunFindAllResponse.getReturnData().getBizObjectArray(), new TypeReference<>() {
            });
            //获取项目总数
            result.parallelStream().forEach(chuanyunGroupProjectDO -> {
                try {
                    // 查询项目号是否在总表中重复
                    findByProjectCode(chuanyunGroupProjectDO.getProjectCode());
                } catch (Exception e) {
                    e.printStackTrace();
                }
            });

        }
        log.info("完成");

    }


    @Test
    public void getDataInMysql() {
        List<ChuanyunGroupProjectDO> all = chuanyunGroupProjectDao.findAll();
        List<String> projectCodeList = all.stream().map(ChuanyunGroupProjectDO::getProjectCode).collect(Collectors.toList()).stream().distinct().collect(Collectors.toList());
        projectCodeList.forEach(projectCode -> {
            try {
                findByProjectCode(projectCode);
            } catch (Exception e) {
                e.printStackTrace();
            }
        });

    }

    // 已经操作了
    private final Vector<String> operated = new Vector<String>();

    private void findByProjectCode(String projectCode) throws Exception {
        //从氚云查询数据
        if (Objects.isNull(projectCode) || projectCode.isBlank()) {
            log.info("projectCode 为空");
            return;
        }
        ArrayList<String> matchers = new ArrayList<>();
        matchers.add("F0000002_2," + projectCode);

        var filter = Filter.instance(0, 10, true, Filter.AND, matchers);
        var chuanyunFindAllResponse = chuanYunManager.findAll(ChuanyunGroupProjectDO.SCHEMA_CODE, filter);
        if (chuanyunFindAllResponse.getReturnData() != null) {
            //转化为POJO
            if (chuanyunFindAllResponse.getReturnData().getTotalCount() > 1) {
                ObjectMapper objectMapper = new ObjectMapper();
                List<ChuanyunGroupProjectDO> result = objectMapper.convertValue(chuanyunFindAllResponse.getReturnData().getBizObjectArray(), new TypeReference<>() {
                });

                for (ChuanyunGroupProjectDO chuanyunGroupProjectDO : result) {
                    if (chuanyunGroupProjectDO.getProjectMemberList() == null || chuanyunGroupProjectDO.getProjectMemberList().length == 0) {
//                        if (Objects.equals(chuanyunGroupProjectDO.getProjectState(), "异常")) {
//                            break;
//                        }
//                        if (!Objects.equals(chuanyunGroupProjectDO.getProjectState(), "已关闭") && !Objects.equals(chuanyunGroupProjectDO.getProjectState(), "异常")) {
//                            String objectId = chuanyunGroupProjectDO.getObjectId();
//                            if (operated.contains(objectId)) {
//                                continue;
//                            }
//                            log.info("objectId: {}, projectName: {}, projectCode: {}", objectId, chuanyunGroupProjectDO.getProjectName(), chuanyunGroupProjectDO.getProjectCode());
//                            log.info("更新项目状态为异常 objectId: {} 之前状态: {}", objectId, chuanyunGroupProjectDO.getProjectState());
//                            boolean b = updateProject(objectId);
//                            if (b) {
//                                operated.add(objectId);
//                            }
//                            break;
//
//                        }
                    }
                }
            }

        }
    }

    /**
     * 更新项目
     */
    private boolean updateProject(String objectId) throws Exception {
        ObjectMapper objectMapper = new ObjectMapper();
        GroupProjectStateUpdateDTO groupProjectStateUpdateDTO = new GroupProjectStateUpdateDTO();
        groupProjectStateUpdateDTO.setProjectState("异常");
        String jsonStr = objectMapper.writeValueAsString(groupProjectStateUpdateDTO);
        ChuanyunSaveDTO update = chuanYunManager.update(ChuanyunGroupProjectDO.SCHEMA_CODE, objectId, jsonStr);
        return update.getSuccessful();
    }


    @Data
    static class GroupProjectStateUpdateDTO {
        /**
         * 字符串表示
         * 未进场
         * 已进场
         * 已上线
         * 进行中
         * 已初验
         * 已终验
         * 已结项
         * 已暂停
         * 已关闭
         * 异常（新增隐藏状态，主要处理重复项目号的问题）
         */
        @JsonProperty("F0000075")
        private String projectState;
    }

    @Test
    public void test1() {

        List<String> objectIdList = Arrays.asList("3785411d-a0c2-11ea-8ae9-7cd30ab711fe",
                "6e948b84-a0c2-11ea-8ae9-7cd30ab711fe",
                "3f7a16d7-a0c2-11ea-8ae9-7cd30ab711fe",
                "d70b2327-54c1-11ec-8d00-7cd30adfe3be",
                "ebe19b1e-85fa-11eb-addb-b8599f3790a4",
                "68758e46-606a-11eb-addb-b8599f3790a4",
                "e8d12167-1410-11eb-addb-b8599f3790a4",
                "dc125f0f-5703-11eb-addb-b8599f3790a4",
                "d65c129f-a632-11ea-9caa-7cd30ae01184",
                "1a1a409d-a637-11ea-9caa-7cd30ae01184",
                "de33d1ef-e122-11ea-addb-b8599f3790a4",
                "fa1eb1e8-e76a-11ea-addb-b8599f3790a4",
                "3f922d1b-8aac-11eb-addb-b8599f3790a4",
                "7807b8f1-cd88-11ea-addb-b8599f3790a4",
                "b0946b54-cd88-11ea-addb-b8599f3790a4",
                "2ee87133-d784-11ea-addb-b8599f3790a4",
                "4ff4aa31-c7f3-11ea-addb-b8599f3790a4",
                "d360b578-a481-11ea-8ae9-7cd30ab711fe",
                "f6182f3a-b70a-11ec-8d00-7cd30adfe3be",
                "c34578d7-9dc5-11ec-8d00-7cd30adfe3be",
                "b8b9fdcf-9b6a-11ec-8d00-7cd30adfe3be",
                "6f94742b-8ab7-11ea-8ae9-7cd30ab711fe",
                "6c2f0329-8ab7-11ea-8ae9-7cd30ab711fe",
                "7f04d493-91bc-11ea-8ae9-7cd30ab711fe",
                "53da5a8a-8ab7-11ea-8ae9-7cd30ab711fe",
                "7ad359aa-1b60-4e34-9be1-0a90a4251701",
                "401a495a-8ab7-11ea-8ae9-7cd30ab711fe",
                "363d5943-8ab7-11ea-8ae9-7cd30ab711fe",
                "31d3908b-8ab7-11ea-8ae9-7cd30ab711fe",
                "2e3018f9-8ab7-11ea-8ae9-7cd30ab711fe");
        objectIdList.forEach(s -> {
            try {
                boolean b = updateProject(s);
                System.out.println(b);
            } catch (Exception e) {
                e.printStackTrace();
            }
        });
    }

    @Test
    public void test2() {
        try {
            findByProjectCode("nullnull");
        } catch (Exception e) {
            e.printStackTrace();
        }
    }
    @Test
    public void test3() {
        try {
            updateProject("2e3018f9-8ab7-11ea-8ae9-7cd30ab711fe");
        } catch (Exception e) {
            e.printStackTrace();
        }
    }
}
