package com.galaxis.manatee.Test;

import com.fasterxml.jackson.core.JsonProcessingException;
import com.fasterxml.jackson.core.type.TypeReference;
import com.fasterxml.jackson.databind.ObjectMapper;
import com.galaxis.manatee.constant.ChuanYunConstant;
import com.galaxis.manatee.dao.*;
import com.galaxis.manatee.entity.chuanyun.data.object.*;
import com.galaxis.manatee.entity.chuanyun.dto.ChuanyunFindAllBizDTO;
import com.galaxis.manatee.entity.chuanyun.dto.Filter;
import com.galaxis.manatee.manager.ChuanYunManager;
import com.galaxis.manatee.service.LogDownloadService;
import com.galaxis.manatee.util.StringUtil;
import com.galaxis.manatee.util.TimeUtil;
import lombok.extern.slf4j.Slf4j;
import org.junit.jupiter.api.Test;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.boot.test.context.SpringBootTest;
import org.springframework.util.StringUtils;

import java.time.Instant;
import java.time.LocalDateTime;
import java.util.ArrayList;
import java.util.List;
import java.util.Objects;
import java.util.Optional;

@Slf4j
@SpringBootTest
public class SaveLog {

    @Autowired
    private ChuanYunManager chuanYunManager;

    @Autowired
    private ChuanyunLogDao chuanyunLogDao;
    @Autowired
    private ChuanyunUserDepartmentNameDao chuanyunUserDepartmentNameDao;
    @Autowired
    private ChuanyunGroupProjectDao chuanyunGroupProjectDao;
    @Autowired
    private ChuanyunUserCompanyDao userCompanyDao;
    @Autowired
    private LogDownloadService logDownloadService;
    private ChuanyunFactoryLocationDao factoryLocationDao;
    private ChuanyunEquipmentProcessDao equipmentProcessDao;

    @Autowired
    public void setFactoryLocationDao(ChuanyunFactoryLocationDao factoryLocationDao) {
        this.factoryLocationDao = factoryLocationDao;
    }

    @Autowired
    public void setEquipmentProcessDao(ChuanyunEquipmentProcessDao equipmentProcessDao) {
        this.equipmentProcessDao = equipmentProcessDao;
    }

    @Test
    public void saveLog() {
        var objectMapper = new ObjectMapper();
        var start = 0;
        var totalCount = 0L;
        var flag = true;
        var pageSize = 100;
        List<String> matchersString = new ArrayList<>();
        //由于需要统计流程中的日志，在日志数据同步时将状态为2的数据一并统计
        matchersString.add("Status_6,[1;2;3]");
        LocalDateTime localDateTimeStart = LocalDateTime.of(2022, 5, 1, 0, 0, 0);
        LocalDateTime localDateTimeEnd = LocalDateTime.of(2022, 5, 30, 23, 59, 59);
        matchersString.add("logDate_1," + TimeUtil.formatDateTime(localDateTimeStart));
        matchersString.add("logDate_3," + TimeUtil.formatDateTime(localDateTimeEnd));
        while (flag) {
            long startTime = Instant.now().getEpochSecond();
            try {
                //从氚云查询数据
                var filter = Filter.instance(start, start + pageSize, true, Filter.AND, matchersString);
                var chuanyunFindAllResponse = chuanYunManager.findAll(ChuanyunLogDO.SCHEMA_CODE, filter);
                if (chuanyunFindAllResponse.getReturnData() == null) {
                    flag = false;
                    continue;
                }
                if (chuanyunFindAllResponse.getReturnData().getTotalCount() >= start + pageSize) {
                    start += pageSize;
                } else {
                    flag = false;
                }

                //转化为POJO
                List<ChuanyunLogDO> result = objectMapper.convertValue(chuanyunFindAllResponse.getReturnData().getBizObjectArray(), new TypeReference<>() {
                });
                //处理日志数据
                result.forEach(this::updateChuanyunLogData);
                //获取项目总数
                totalCount = chuanyunFindAllResponse.getReturnData().getTotalCount();
                //保存
                chuanyunLogDao.saveAll(result);
            } catch (Exception e) {
                log.error(e.getMessage());
                e.printStackTrace();
            }
            log.info("日志100条保存时间" + (Instant.now().getEpochSecond() - startTime) + "秒");
        }
        log.info("最近一月历史日志信息保存" + totalCount);
    }

    /**
     * 更新氚云日志数据
     *
     * @param chuanyunLogDO 待处理数据
     */
    private void updateChuanyunLogData(ChuanyunLogDO chuanyunLogDO) {
        //处理项目userId
        if (chuanyunLogDO.getUserObject() != null) {
            chuanyunLogDO.setUserId(chuanyunLogDO.getUserObject().getObjectId());
        }
        //日志人员所属部门名称
        if (!StringUtils.isEmpty(chuanyunLogDO.getDepartment())) {
            chuanyunLogDO.setDepartmentName(chuanyunLogDO.getDepartment());
        } else {
            chuanyunLogDao.findById(chuanyunLogDO.getObjectId()).ifPresent(oldLogDO -> {
                chuanyunLogDO.setDepartmentName(oldLogDO.getDepartmentName());
            });
        }

        if (Objects.isNull(chuanyunLogDO.getDepartmentName()) || chuanyunLogDO.getDepartmentName().isBlank()) {
            // 任没有部门，查询该用户所属公司获取部门
            Optional<ChuanyunUserCompanyDO> first = userCompanyDao.findAllByUserIdOrderByModifiedTimeDesc(chuanyunLogDO.getUserId()).stream().findFirst();
            first.ifPresent(chuanyunUserCompanyDO -> {
                String[] split = chuanyunUserCompanyDO.getDepartment().split("/");
                int length = split.length;
                chuanyunLogDO.setDepartmentName(split[length - 1]);
            });
        }
        //处理数据
        if (chuanyunLogDO.getDayLogList() == null || chuanyunLogDO.getDayLogList().size() == 0) {
            chuanyunLogDO.setDayLogList(new ArrayList<>());
            log.warn(chuanyunLogDO + "");
            log.warn("异常数据");
        } else {
            //如果有重复就删除
            chuanyunLogDao.findById(chuanyunLogDO.getObjectId()).ifPresent(chuanyunLogDao::delete);
            //根据projectId获取code,如果是生产项目，则不用处理
            //针对前期测试数据，可能出现日志为空的情况，跳过不处理
            // fixme 每周工时没有工作组id
            chuanyunLogDO.getDayLogList().forEach(chuanyunDayLog -> {
                // 补全工作组
                if (ChuanYunConstant.LOG_WEEK_TYPE.contains(chuanyunLogDO.getDayOrWeek()) && ChuanYunConstant.NEED_WORK_GROUP_PROJECT_TYPE.contains(chuanyunDayLog.getProjectType())) {
                    List<ChuanyunWeekLogDO> weekLogList = chuanyunLogDO.getWeekLogList();
                    for (ChuanyunWeekLogDO chuanyunWeekLogDO : weekLogList) {
                        if (Objects.equals(chuanyunDayLog.getProjectId(), chuanyunWeekLogDO.getProjectId())) {
                            chuanyunDayLog.setWorkGroupId(chuanyunWeekLogDO.getWorkGroupId());
                            break;
                        }
                    }
                }

                //离职人员补全用户Id及部门ID
                if (StringUtils.isEmpty(chuanyunLogDO.getUserId())) {
                    var name = "";
                    if (StringUtils.isEmpty(chuanyunDayLog.getContent())) {
                        name = chuanyunLogDO.getName();
                    } else {
                        name = chuanyunDayLog.getContent();
                    }
                    chuanyunLogDO.setUserId(StringUtil.getCharacters(name));
                    ChuanyunUserDepartmentNameDO chuanyunUserDepartmentNameDO = chuanyunUserDepartmentNameDao.findByUserName(chuanyunLogDO.getUserId());
                    if (chuanyunUserDepartmentNameDO != null) {
                        chuanyunLogDO.setDepartmentName(chuanyunUserDepartmentNameDO.getDepartmentName());
                    }
                }
                try {
                    if (!ChuanYunConstant.PRODUCTION_PROJECT.equals(chuanyunDayLog.getProjectType()) && StringUtils.isEmpty(chuanyunDayLog.getProjectId())) {
                        log.info(chuanyunLogDO + "");
                    } else if (!ChuanYunConstant.PRODUCTION_PROJECT.equals(chuanyunDayLog.getProjectType())) {
                        ChuanyunGroupProjectDO chuanyunGroupProjectDO = chuanyunGroupProjectDao.findById(chuanyunDayLog.getProjectId()).orElseThrow();
                        chuanyunDayLog.setProjectCode(chuanyunGroupProjectDO.getProjectCode());
                    }
                } catch (Exception e) {
                    e.printStackTrace();
                    log.warn(chuanyunDayLog + "");
                }
            });
        }
    }

    @Test
    public void saveLog2() {
        logDownloadService.saveDayLog();
    }

    @Test
    public void saveFactoryLocation() {
        //从氚云查询数据
        var filter = Filter.instance(0, 20, true);
        ChuanyunFindAllBizDTO<Object> chuanyunFindAllResponse = null;
        try {
            chuanyunFindAllResponse = chuanYunManager.findAll("D0017896651391eded64eabb0e41ce458c4192d", filter);
        } catch (JsonProcessingException e) {
            e.printStackTrace();
        }
        ObjectMapper objectMapper = new ObjectMapper();
        //转化为POJO
        List<ChuanyunFactoryLocation> result = objectMapper.convertValue(chuanyunFindAllResponse.getReturnData().getBizObjectArray(), new TypeReference<>() {
        });
        //保存
        factoryLocationDao.saveAll(result);
    }

    @Test
    public void saveEquipmentProcess() {
        //从氚云查询数据
        var filter = Filter.instance(0, 100, true);
        ChuanyunFindAllBizDTO<Object> chuanyunFindAllResponse = null;
        try {
            chuanyunFindAllResponse = chuanYunManager.findAll("D0017896b29261f8b404521b0a404d79ea4db04", filter);
        } catch (JsonProcessingException e) {
            e.printStackTrace();
        }
        ObjectMapper objectMapper = new ObjectMapper();
        //转化为POJO
        List<ChuanyunEquipmentProcess> result = objectMapper.convertValue(chuanyunFindAllResponse.getReturnData().getBizObjectArray(), new TypeReference<>() {
        });
        //保存
        equipmentProcessDao.saveAll(result);
    }


}
